/*
 *  Copyright 2001-2005 Internet2
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

 /* SAMLStatement.cpp - SAML statement implementation

   Scott Cantor
   5/22/02

   $History:$
*/

#include "internal.h"

using namespace saml;
using namespace std;

SAMLStatement::SAMLStatementFactoryMap SAMLStatement::m_map;

SAMLStatement* SAMLStatement::getInstance(DOMElement* e)
{
    saml::QName* type=saml::QName::getQNameAttribute(e,XML::XSI_NS,L(type));
    if (!type)
        type=new saml::QName(e->getNamespaceURI(),e->getLocalName());
    SAMLStatementFactoryMap::const_iterator i=m_map.find(*type);
    delete type;
    if (i!=m_map.end())
        return (i->second)(e);
    return NULL;
}

SAMLStatement* SAMLStatement::getInstance(istream& in)
{
    XML::Parser p;
    XML::StreamInputSource src(in);
    Wrapper4InputSource dsrc(&src,false);
    DOMDocument* doc=p.parse(dsrc);
    try
    {
        SAMLStatement* s=getInstance(doc->getDocumentElement());
        if (s)
            s->setDocument(doc);
        return s;
    }
    catch(...)
    {
        doc->release();
        throw;
    }
}
